/*
 * Example that shows how to use RTC & NTP with a TZ timezone string.
 *
 * Recommended use:
 *	Bring up Serial monitor at 115200 *before* uploading
 *	Upload code and observe results
 *
 * Code is realeased to the public domain
 * 2018-11-03  Bill Perry
*/

#include <ESP8266WiFi.h>
#include <time.h>                       // time() ctime()
#include <sys/time.h>                   // struct timeval
#include <coredecls.h>                  // settimeofday_cb()


////////////////////////////////////////////////////////
//
// WiFi setup
//
#include <WiFiManager.h>

#define SSID            "your-ssid"
#define SSIDPWD         "your-ssid_password"

////////////////////////////////////////////////////////

timeval cbtime;			// when time set callback was called
bool cbtime_set = false;

// callback function that is called whenever time is set
// could potentially be used to update RTC
void time_is_set (void)
{
  gettimeofday(&cbtime, NULL);
  cbtime_set = true;
  Serial.println("------------------ settimeofday() was called ------------------");
}

void setup() {
  Serial.begin(115200);

  for(int i = 0; i < 10; i++)
  {
	printf("in setup\n");
	delay(500);
  }

  // set function to call when time is set
  // is called by NTP code when NTP is used
  settimeofday_cb(time_is_set);

  // set time from RTC
  // Normally you would read the RTC to eventually get a current UTC time_t
  // this is faked for now.
  time_t rtc_time_t = 1541267183; // fake RTC time for now

  timezone tz = { 0, 0};
  timeval tv = { rtc_time_t, 0};

  // DO NOT attempt to use the timezone offsets
  // The timezone offset code is really broken.
  // if used, then localtime() and gmtime() won't work correctly.
  // always set the timezone offsets to zero and use a proper TZ string
  // to get timezone and DST support.

  // set the time of day and explicitly set the timezone offsets to zero
  // as there appears to be a default offset compiled in that needs to
  // be set to zero to disable it.
  settimeofday(&tv, &tz);


  // set up TZ string to use a POSIX/gnu TZ string for local timezone
  // TZ string information:
  // https://www.gnu.org/software/libc/manual/html_node/TZ-Variable.html
  setenv("TZ", "CST+6CDT,M3.2.0/2,M11.1.0/2", 1);

  tzset(); // save the TZ variable

  // turn on WIFI using SSID/SSIDPWD
  WiFi.mode(WIFI_STA);
  WiFi.begin(SSID, SSIDPWD);

  // enable NTP by setting up NTP server(s)
  // up to 3 ntp servers can be specified
  // configTime(tzoffset, dstflg, "ntp-server1", "ntp-server2", "ntp-server3");
  // set both timezone offet and dst parameters to zero 
  // and get real timezone & DST support by using a TZ string
  configTime(0, 0, "pool.ntp.org");

  // don't wait, observe time changing when ntp timestamp is received

  Serial.println();

}

// for testing purpose:
extern "C" int clock_gettime(clockid_t unused, struct timespec *tp);

#define PTM(w) \
  Serial.print(":" #w "="); \
  Serial.print(tm->tm_##w);

void printTm (const char* what, const tm* tm)
{
  Serial.print(what);
  PTM(isdst); PTM(yday); PTM(wday);
  PTM(year);  PTM(mon);  PTM(mday);
  PTM(hour);  PTM(min);  PTM(sec);
}

timeval tv;
struct timezone tz;
timespec tp;
time_t tnow;

void loop()
{

  gettimeofday(&tv, &tz);
  clock_gettime(0, &tp); // also supported by esp8266 code
  tnow = time(nullptr);

  // localtime / gmtime every second change
  static time_t lastv = 0;
  if (lastv != tv.tv_sec)
  {
    lastv = tv.tv_sec;
#if 1
  printf("tz_minuteswest: %d, tz_dsttime: %d\n", tz.tz_minuteswest, tz.tz_dsttime);
  printf("gettimeofday() tv.tv_sec : %ld\n", lastv);
  printf("time()            time_t : %ld\n", tnow);
    Serial.println();
#endif

    printf("         ctime: %s", ctime(&tnow)); // print formated local time
    printf(" local asctime: %s", asctime(localtime(&tnow))); // print formated local time
    printf("gmtime asctime: %s", asctime(gmtime(&tnow))); // print formated gm time
	
    // print gmtime and localtime tm members
    printTm("      gmtime", gmtime(&tnow));
    Serial.println();
    printTm("   localtime", localtime(&tnow));
    Serial.println();

    Serial.println();
  }
  delay(100);
}
